const AWSLambdaCaller = require('./services/AWSLambdaCaller');
const Util = require('./util/Util');
const DialogFlowAccessor = require('./services/DialogFlowAccessor');
const KeyMatcherAccessor = require('./services/KeyMatcherAccessor');
const RasaAccessor = require('./services/RasaAccessor');

class AsksuiteProcessingNatural {
    static get ORDERED_ACCESSORS() { return ['DIALOG_FLOW', 'RASA', 'KEY_MATCHER']; } // Robot will search for intents in this order

    constructor() {
    };

    findIntent(request) {

        return new  Promise(async (resolve, reject) => {
            let intent = {};

            const awsLambdaCaller = new AWSLambdaCaller(request.config.CORE_LAMBDA_AWS);
            const dialogFlowAccessor = new DialogFlowAccessor(awsLambdaCaller);
            const keyMatcherAccessor = new KeyMatcherAccessor(awsLambdaCaller);
            const rasaAccessor = new RasaAccessor(request.config.RASA_URL, request.config.RASA_CONFIDENCE_THRESHOLD);

            for (const accessorName of AsksuiteProcessingNatural.ORDERED_ACCESSORS) {
                switch (accessorName) {
                    case 'DIALOG_FLOW':
                        intent = await dialogFlowAccessor.resolveText(request);
                        break;
                    case 'KEY_MATCHER':
                        intent = await keyMatcherAccessor.resolveText(request);
                        intent.keywordTry = { try: true };
                        break;
                    case 'RASA':
                        intent = await rasaAccessor.resolveText(request);
                        break;
                }

                request.findIntent = intent;

                if (intent.intent && !Util.isNaoentendi(intent.intent)) {
                    intent.resolver = accessorName;

                    if (intent.resolver === 'KEY_MATCHER') {
                        intent.keywordTry.match = intent.intent;
                        intent.keywordTry.intent = intent.intent;
                    }

                    //FOR exit condition
                    break;
                }

            }

            resolve(intent);

        })

    }
}

module.exports = AsksuiteProcessingNatural;
